#! /usr/bin/python
# -*- coding: utf-8 -*-
#
#    dbus_support.py - Provides a dbus service to control the application.
#
#    Copyright  2005-07 Adolfo González Blázquez <code@infinicode.org>
#               2011 PhobosK <phobosk@kbfx.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#    MA 02110-1301, USA.

"""Provides a dbus service to control the application.

Creates a "Controller" dbus object to control the aplication.
When the application is started normally this object has a "ShowPreferences"
method to show the preferences of the application if another application is
concurrently run with the -p command line option. The object has a method
"isNested" that returns boolean, and this method is missing when the application
is run with the -p command line option.
When the application is started with the -p command line option the object has
an "Exit" and a "PresentPreferences" methods to avoid running together with
another instances.

Interacts with the main application via the "camera" and "prefs_alone" objects.
These objects are set by the main application.
"""

import gtk
import gobject
import dbus
import dbus.service
import dbus.mainloop.glib

from cameramonitor import camglobals as glob

__authors__ = "Adolfo González Blázquez <code@infinicode.org>, PhobosK <phobosk@kbfx.net>"
__copyright__ = "Copyright 2005-11, Adolfo González Blázquez and PhobosK"
__license__ = "GPL"
__version__ = "0.3.2"
__maintainer__ = "PhobosK <phobosk@kbfx.net>"
__status__ = "Development"

debug = 0


class DBUSCameramonitorControl(dbus.service.Object):
    """Creates the needed dbus methods for the dbus "Controller" object
    when the application is started normally.

    Interacts with the main application via the "camera" object.
    This object is set by the main application.
    """

    @dbus.service.method('com.CameraMonitor.ControlInterface',
                         in_signature='', out_signature='s')
    def ShowPreferences(self):
        """Creates and shows the preferences window of the application.

        @return: Returns the message "Preference dialog shown." to the callee.
        @rtype: String
        """
        camera.prefs.preferences_show(None)
        if debug: print "[DBus]: Preference dialog shown."
        return 'DBUS: Preference dialog shown.'

    @dbus.service.method("com.CameraMonitor.ControlInterface",
                         in_signature='', out_signature='b')
    def isNested(self):
        """Used to check if the application is running normally.

        @return: Returns Boolean to the callee telling it that this is nested.
        @rtype: Bool
        """
        if debug: print "[DBus]: Checked if application is run normally."
        return True

    @dbus.service.method("com.CameraMonitor.ControlInterface",
                         in_signature='', out_signature='s')
    def Exit(self):
        gtk.main_quit()
        if debug: print "[DBus]: Exiting %s." % glob.name_long
        return "Exiting %s." % glob.name_long


class DBUSCameramonitorPreferences(dbus.service.Object):
    """Creates the dbus methods for the dbus "Controller" object when the
    application is started with the -p command line option.

    Interacts with the main application via the "prefs_alone" object.
    This object is set by the main application.
    """

    @dbus.service.method('com.CameraMonitor.ControlInterface',
                         in_signature='', out_signature='s')
    def PresentPreferences(self):
        """Presents to the user the already opened preference window.

        @return: Returns the message "Preference dialog presented." to the callee.
        @rtype: String
        """
        prefs_alone.prefs.present()
        if debug: print "[DBus]: Preference dialog presented."
        return 'DBus: Preference dialog presented.'

    @dbus.service.method("com.CameraMonitor.ControlInterface",
                         in_signature='', out_signature='s')
    def Exit(self):
        gtk.main_quit()
        if debug: print "[DBus]: exiting %s's preferences dialog." % glob.name_long
        return "Exiting %s's preferences dialog." % glob.name_long


class DbusController():
    """Creates the dbus "Controller" object.

    Which methods will be created depends on the command line option -p, i.e. on
    how the application was started.
    """

    def __init__(self, object_string = "Controller"):
        """Connects to the session DBus and creates the necessary object.

        @param object_string: The object type that should be built.
                                Valid are: "Controller" or "Preferences"
        @type object_string: String.
        """
        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

        self.session_bus = dbus.SessionBus()
        self.service_name = dbus.service.BusName("com.CameraMonitor", self.session_bus)
        if object_string == "Controller":
            self.service_object = DBUSCameramonitorControl(self.session_bus, \
                                                "/com/CameraMonitor/Controller")
        else:
            self.service_object = DBUSCameramonitorPreferences(self.session_bus, \
                                                "/com/CameraMonitor/Controller")

        if debug: print "[DBus]: Starting %s dbus '%s' service." % (glob.name_long, \
                                                                    object_string)
