#! /usr/bin/python
# -*- coding: utf-8 -*-
#
#    preferences.py - Manages application's preferences.
#
#    Copyright  2005-07 Adolfo González Blázquez <code@infinicode.org>
#               2011 PhobosK <phobosk@kbfx.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#    MA 02110-1301, USA.

"""Manages application's preferences.

    - Creates and updates the application preferences.
    - Watches changes via gconf and takes appropriate actions.
    - Creates the application's preferences dialog.'
"""

import pygtk
pygtk.require("2.0")
import gtk
import gconf

from cameramonitor import camglobals as glob
from cameramonitor import autostart
from cameramonitor import utils

__authors__ = "Adolfo González Blázquez <code@infinicode.org>, PhobosK <phobosk@kbfx.net>"
__copyright__ = "Copyright 2005-11, Adolfo González Blázquez and PhobosK"
__license__ = "GPL"
__version__ = "0.3.2"
__maintainer__ = "PhobosK <phobosk@kbfx.net>"
__status__ = "Development"


class CameraMonitorPreferences:
    """Takes care of managing gconf preferences and the preferences window."""

    def __init__(self, state):
        """Initializes the prefs in gconf and creates the preferences window.

        @param state: If this is a preferences alone dialog, or the whole
                        application. Valid are: "alone" and "nested".
        @type state: String
        """
        self.state = state

        self.video_device = glob.video_device_default
        self.icon_visible = glob.icon_visible_default
        self.notify = glob.notify_default
        self.autostart = glob.autostart_default

        self.preferences_get()

        self.prefs = self.preferences_create()

        # Monitor the keys for changes.
        self.gconf_monitor = utils.preferences_monitor_gconf()
        self.gconf_monitor.add_key(self.preferences_video_key_changed)
        self.gconf_monitor.add_key(self.preferences_icon_visible_key_changed, \
                                                        glob.gconf_key_icon_visible)
        self.gconf_monitor.add_key(self.preferences_notify_key_changed, \
                                                        glob.gconf_key_notify)
        self.gconf_monitor.add_key(self.preferences_autostart_key_changed, \
                                                        glob.gconf_key_autostart)


    def preferences_get(self):
        """Reads and sets preferences using GConf."""
        client = gconf.client_get_default()

        video = client.get_string(glob.gconf_key_video_device)
        if video != None: self.video_device = video
        else: client.set_string(glob.gconf_key_video_device, self.video_device)

        icon_visible = client.get_bool(glob.gconf_key_icon_visible)
        if icon_visible != None: self.icon_visible = icon_visible
        else: client.set_bool(glob.gconf_key_icon_visible, self.icon_visible)

        notify = client.get_bool(glob.gconf_key_notify)
        if notify != None: self.notify = notify
        else: client.set_bool(glob.gconf_key_notify, self.notify)

        autostart = client.get_bool(glob.gconf_key_autostart)
        if autostart != None: self.autostart = autostart
        else: client.set_bool(glob.gconf_key_autostart, self.autostart)


    def preferences_save_video_device(self):
        """Saves the video device path."""
        client = gconf.client_get_default()
        client.set_string(glob.gconf_key_video_device, self.video_device)

    def preferences_save_icon_visible(self):
        """Saves the icon_visible preferences."""
        client = gconf.client_get_default()
        client.set_bool(glob.gconf_key_icon_visible, self.icon_visible)

    def preferences_save_notify(self):
        """Saves the notification preferences."""
        client = gconf.client_get_default()
        client.set_bool(glob.gconf_key_notify, self.notify)

    def preferences_save_autostart(self):
        """Saves the autostart preferences."""
        client = gconf.client_get_default()
        client.set_bool(glob.gconf_key_autostart, self.autostart)


    def device_change_end(self, widget, event):
        """Video device change finished.

        Checks if something is really changed or it is just lost focus.
        Thus avoiding unnecessary updates of the watcher thread.
        """
        changed_to = widget.get_text()
        if changed_to == self.video_device: return
        self.video_device = changed_to
        self.preferences_save_video_device()

    def icon_visible_changed(self, widget):
        """Icon visibility changed."""
        self.icon_visible = widget.get_active()
        self.preferences_save_icon_visible()

    def notify_changed(self, widget):
        """Notify changed."""
        self.notify = widget.get_active()
        self.preferences_save_notify()

    def autostart_changed(self, widget):
        """Autostart checkbox changed.

        Before saving the value check if the autostart file is really
        added/removed successfully.
        """
        self.autostart = widget.get_active()
        if self.autostart:
            if not autostart.create_autostart_file():
                self.autostart_check.set_active(False)
        else:
            if not autostart.delete_autostart_file():
                self.autostart_check.set_active(True)

        self.preferences_save_autostart()


    def preferences_video_key_changed(self, client, connection_id, entry, args):
        """Callback: someone's changed the gconf key for video_device."""
        self.video_device = entry.get_value().get_string()
        self.dev_entry.set_text(self.video_device)

    def preferences_icon_visible_key_changed(self, client, connection_id, entry, args):
        """Callback: someone's changed the gconf key for visibility of icon."""
        self.icon_visible = entry.get_value().get_bool()
        self.icon_visible_check.set_active(self.icon_visible)

    def preferences_notify_key_changed(self, client, connection_id, entry, args):
        """Callback: someone's changed the gconf key for notification."""
        self.notify = entry.get_value().get_bool()
        self.notify_check.set_active(self.notify)

    def preferences_autostart_key_changed(self, client, connection_id, entry, args):
        """Callback: someone's changed the gconf key for autostart."""
        self.autostart = entry.get_value().get_bool()
        self.autostart_check.set_active(self.autostart)


    def preferences_close_clicked(self, widget):
        """Callback: button close pressed on preferences or window closed.

        No need to save all prefs 'cause they are already saved.'"""
        self.prefs.destroy()
        if self.state == 'alone': gtk.main_quit()


    def preferences_show(self, widget):
        """Callback: display preferences dialog."""
        if self.prefs.get_children() == []: self.prefs = self.preferences_create()
        else: self.prefs.present()

        self.dev_entry.set_text(self.video_device)
        self.icon_visible_check.set_active(self.icon_visible)
        self.notify_check.set_active(self.notify)
        self.autostart_check.set_active(self.autostart)
        self.prefs.show_all()

        if self.autostart:
            if not autostart.create_autostart_file():
                self.autostart_check.set_active(False)
        else:
            if not autostart.delete_autostart_file():
                self.autostart_check.set_active(True)


    def preferences_create(self):
        """Creates the preferences dialog."""
        prefs = gtk.Window()
        prefs.set_title(glob.name_long + " " + glob.title_preferences)
        prefs.set_icon_from_file(glob.image["on"])
        prefs.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        prefs.set_resizable(False)
        prefs.connect("destroy", self.preferences_close_clicked)

        # Video Device Prefs
        dev_desc = gtk.Label("<b>" + glob.label_device_path + "</b>")
        dev_desc.set_use_markup(True)
        dev_desc.set_alignment(xalign=0, yalign=0.5)

        dev_label = gtk.Label(glob.message_video_device)
        dev_label.set_alignment(xalign=0, yalign=0.5)
        self.dev_entry = gtk.Entry()
        self.dev_entry.set_text(self.video_device)
        self.dev_entry.set_activates_default(True)
        self.dev_entry.connect('focus-out-event',self.device_change_end)

        dev_hbox = gtk.HBox()
        dev_hbox.set_spacing(10)
        dev_hbox.pack_start(dev_label)
        dev_hbox.pack_start(self.dev_entry)

        dev_vbox = gtk.VBox()
        dev_vbox.set_border_width(10)
        dev_vbox.set_spacing(10)
        dev_vbox.pack_start(dev_desc)
        dev_vbox.pack_start(dev_hbox)

        # Icon visibility Prefs
        icon_visible_desc = gtk.Label("<b>" + glob.label_visibility + "</b>")
        icon_visible_desc.set_use_markup(True)
        icon_visible_desc.set_alignment(xalign=0, yalign=0.5)

        self.icon_visible_check = gtk.CheckButton(label=glob.label_show_tray)
        self.icon_visible_check.set_alignment(xalign=0, yalign=0.5)
        self.icon_visible_check.connect('toggled',self.icon_visible_changed)

        icon_visible_vbox = gtk.VBox()
        icon_visible_vbox.set_border_width(10)
        icon_visible_vbox.set_spacing(10)
        icon_visible_vbox.pack_start(icon_visible_desc)
        icon_visible_vbox.pack_start(self.icon_visible_check)

        # Notification Prefs
        notify_desc = gtk.Label("<b>" + glob.label_notify + "</b>")
        notify_desc.set_use_markup(True)
        notify_desc.set_alignment(xalign=0, yalign=0.5)

        self.notify_check = gtk.CheckButton(label=glob.label_always_notify)
        self.notify_check.set_alignment(xalign=0, yalign=0.5)
        self.notify_check.connect('toggled',self.notify_changed)

        notify_vbox = gtk.VBox()
        notify_vbox.set_border_width(10)
        notify_vbox.set_spacing(10)
        notify_vbox.pack_start(notify_desc)
        notify_vbox.pack_start(self.notify_check)

        # Autostart Prefs
        autostart_desc = gtk.Label("<b>" + glob.label_autostart + "</b>")
        autostart_desc.set_use_markup(True)
        autostart_desc.set_alignment(xalign=0, yalign=0.5)

        self.autostart_check = gtk.CheckButton(label=glob.label_always_autostart)
        self.autostart_check.set_alignment(xalign=0, yalign=0.5)
        self.autostart_check.connect('toggled',self.autostart_changed)

        autostart_vbox = gtk.VBox()
        autostart_vbox.set_border_width(10)
        autostart_vbox.set_spacing(10)
        autostart_vbox.pack_start(autostart_desc)
        autostart_vbox.pack_start(self.autostart_check)

        # OK and Cancel buttons
        closebutton = gtk.Button(stock=gtk.STOCK_CLOSE)
        closebutton.connect('clicked', self.preferences_close_clicked)
        closebutton.set_flags(gtk.CAN_DEFAULT)

        buttonbox = gtk.HButtonBox()
        buttonbox.set_spacing(10)
        buttonbox.set_layout(gtk.BUTTONBOX_END)
        buttonbox.pack_start(closebutton)

        # Packing everythin into a vbox
        vbox = gtk.VBox()
        vbox.set_border_width(10)
        vbox.set_spacing(10)
        vbox.pack_start(dev_vbox)
        vbox.pack_start(icon_visible_vbox)
        vbox.pack_start(notify_vbox)
        vbox.pack_start(autostart_vbox)
        vbox.pack_start(buttonbox)

        prefs.add(vbox)
        self.dev_entry.grab_focus()
        closebutton.grab_default()

        return prefs


if __name__ == "__main__":
    print "Description: \n%s" % __doc__
    print "Version: %s" % __version__
    print "Authors: %s" % __authors__
