#! /usr/bin/python
# -*- coding: utf-8 -*-
#
#    utils.py - Holds the common utils used by the package.
#
#    Copyright  2005-07 Adolfo González Blázquez <code@infinicode.org>
#               2011 PhobosK <phobosk@kbfx.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#    MA 02110-1301, USA.

"""Holds some common utilities used by the package.

The utilities include:
    - GConf key monitor class
    - check of the video device path
    - management of the application instances
    - management of the warning/error dialogs
"""

import os
import errno
from traceback import print_exc

import pygtk
pygtk.require("2.0")
import gtk
import dbus
import gconf

from cameramonitor import camglobals as glob

__authors__ = "Adolfo González Blázquez <code@infinicode.org>, PhobosK <phobosk@kbfx.net>"
__copyright__ = "Copyright 2005-11, Adolfo González Blázquez and PhobosK"
__license__ = "GPL"
__version__ = "0.3.2"
__maintainer__ = "PhobosK <phobosk@kbfx.net>"
__status__ = "Development"

debug = 0
about_shown = False


class preferences_monitor_gconf():
    """Monitors GConf keys for changes."""

    def __init__(self, path = glob.gconf_path):
        """Initialises the GConf client.

        @param path: the GConf path of the application.
        @type path: String
        """
        self.client = gconf.client_get_default()
        self.client.add_dir(path, gconf.CLIENT_PRELOAD_NONE)

    def add_key(self, callback, key = glob.gconf_key_video_device):
        """Adds a key to the client for monitoring.

        @param callback: reference to callback function.

        @param key: the GConf key to watch. Defaults to video device key.
        @type key: String

        @return: notification id needed to destroy the notification later.
        """
        notify_id = self.client.notify_add(key, callback)
        return notify_id

    def remove_key(self, notify_id):
        """Removes a key from the client for monitoring.

        @param notify_id: notification id. If 0 means we had an error in GConf.
        """
        if notify_id: self.client.notify_remove(notify_id)


def check_perms(device):
        """Checks the access permissions of the device to watch.

        @param device: The path to the device to be watched.
        @type device: String
        @return: "errno.EACCES" on access restrictions, broken symlinks and on dirs.
                 "1" if it can still monitor the device though the restrictions.
                 Latter covers cases when the device is readable but its directory
                 (or file) has the "x" for others attribute unset, i.e o-x.
        @rtype: Integer
        """
        if os.path.isdir(device):
            return errno.EACCES # We do not watch directories.
        if os.path.islink(device) and not os.path.exists(device):
            return errno.EACCES # We do not watch broken symlinks.

        try:
                fd = open(device)
        except IOError as e:
                if e.errno == errno.ENOENT:
                    return errno.ENOENT # Device doesn't exist yet.
                if e.errno == errno.EACCES:
                        if os.access(os.path.dirname(device), os.R_OK):
                            return 1 # No permissions but still we can watch it.
                        return errno.EACCES # No path perms to read from device.

                # Raise other errors.
                raise
        else:
                fd.close()


def is_unique_instance(instance = "nested"):
    """Checks if this instance of the application is unique.

    @param instance: Shows who is doing the check. Valid are: "nested" or "alone".
    @type instance: String.

    @return: A Boolean showing if it is unique or already running.
    @rtype: Bool.
    """
    bus = dbus.SessionBus()

    try:
        remote_object = bus.get_object("com.CameraMonitor",
                                       "/com/CameraMonitor/Controller")
    except dbus.DBusException:
        if debug: print_exc()
        bus.close()
        return True

    try:
        method_reply = remote_object.isNested( \
                        dbus_interface = "com.CameraMonitor.ControlInterface")
    except:
        if debug: print_exc()
        if instance == "alone":
            method_reply = remote_object.PresentPreferences( \
                        dbus_interface = "com.CameraMonitor.ControlInterface")
            bus.close()
            return False
        else:
            method_reply = remote_object.Exit( \
                        dbus_interface = "com.CameraMonitor.ControlInterface")
            bus.close()
            return True

    method_reply = remote_object.ShowPreferences( \
            dbus_interface = "com.CameraMonitor.ControlInterface")
    bus.close()
    return False


def display_dialog_error(text):
    """Displays an error dialog that is marked "always on top" and has markup on.

    @param text: Holds the text to be shown. Can be a markup too.
    @type text: String
    """
    dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, None)
    dialog.set_icon_from_file(glob.logo)
    dialog.set_title(glob.name_long)
    dialog.set_markup(text)
    dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
    dialog.set_keep_above(True)
    dialog.set_modal(True)
    dialog.connect('response', dialog_destroy)
    dialog.show()


def display_dialog_warning(text):
    """Displays a warning dialog that is marked "always on top" and has markup on.

    @param text: Holds the text to be shown. Can be a markup too.
    @type text: String
    """
    dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, None)
    dialog.set_icon_from_file(glob.logo)
    dialog.set_title(glob.name_long)
    dialog.set_markup(text)
    dialog.set_position(gtk.WIN_POS_CENTER_ALWAYS)
    dialog.set_keep_above(True)
    dialog.set_modal(True)
    dialog.connect('response', dialog_destroy)
    dialog.show()


def dialog_destroy(dialog, response_id):
    """Destroys the dialog and acts according to response id.

    Used to avoid creating of another mainloop by gtk.Dialog.run().
    If it is an "About" dialog, clears the global about_shown.
    """
    global about_shown

    if dialog.get_name() == glob.name_long + ' '+ glob.version:
        about_shown = False

    dialog.destroy()


if __name__ == "__main__":
    print "Description: \n%s" % __doc__
    print "Version: %s" % __version__
    print "Authors: %s" % __authors__
