#! /usr/bin/python
# -*- coding: utf-8 -*-
#
#    watch.py - Watches a transient file for changes using inotify.
#
#    Copyright  2005-07 Adolfo González Blázquez <code@infinicode.org>
#               2011 PhobosK <phobosk@kbfx.net>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#    MA 02110-1301, USA.

"""Monitors a transient file for changes.

Monitors a transient file for changes using inotify and signals main application
when a change occur.
Interacts with the main application via the "camera" object. This object is set
by the main application.
"""

import sys

import gtk
import pyinotify

from cameramonitor import utils

__authors__ = "Adolfo González Blázquez <code@infinicode.org>, PhobosK <phobosk@kbfx.net>"
__copyright__ = "Copyright 2005-11, Adolfo González Blázquez and PhobosK"
__license__ = "GPL"
__version__ = "0.3.2"
__maintainer__ = "PhobosK <phobosk@kbfx.net>"
__status__ = "Development"

debug = 0


class CameraMonitorWatcher:
    """Creates inotify watcher for the video device.

    The watcher starts in a new thread and handles the changes through
    HandleEvents subclass.
    Can't watch directories, broken symlinks and files residing in paths
    with no access permissions granted, so be sure to make the necessary checks
    before using the update() method of this class. For the checks use a
    utils.check_perms(device_path) call.

    @warning: The watcher cannot monitor the changes of the target of a symlink.
              See the TODO field.

    @todo: Add a notifier for changed target of a watched symlink. Probably will
           be done when this is available in pyinotify future versions.
    """

    def __init__(self, device):
        """Starts the watcher thread and adds device to the watch.

        @param device: The path to the device to be watched.
        @type device: String
        """
        if debug:
            self.mask = pyinotify.ALL_EVENTS
        else:
            self.mask = pyinotify.IN_MOVED_FROM | pyinotify.IN_MOVED_TO | \
                        pyinotify.IN_CLOSE_WRITE | pyinotify.IN_CLOSE_NOWRITE | \
                        pyinotify.IN_DELETE_SELF | pyinotify.IN_OPEN

        self.watch_manager = pyinotify.WatchManager()
        self.notifier = pyinotify.ThreadedNotifier(self.watch_manager)
        self.notifier.start()
        if debug: print "[watch]: Watcher thread started."

        self.old_watch_path = ""
        self.watch_manager_dict = {}
        self.update(device)

    def update(self, device):
        """Adds or replaces the video device the watcher is monitoring.

        Rejects directories, broken symlinks and files residing in paths with no
        access permissions granted, so before passing a path, check it for these
        conditions. For the check use a utils.check_perms(device_path) call.

        @param device: The path to the device to be watched.
        @type device: String
        """
        self.unicode_path(device)
        if device == self.old_watch_path:
            if debug: print "[watch]: Device path is the same: %s . Not updating watcher." % device
            return
        self.old_watch_path = device
        if self.watch_manager.watches:
            self.watch_manager.rm_watch(self.watch_manager_dict.values())
            if debug: print "[watch]: Removing old watch."
        if not camera.no_access:
            self.watch_manager_dict = self.watch_manager.watch_transient_file(self.path, self.mask, HandleEvents)
            if debug: print "[watch]: Watcher updated with video device: %s" % self.path
        else:
            if debug: print "[watch]: No access to the video device %s . Won't watch." \
                                                                    % self.path

    def stop(self):
        """Stops the watcher thread on exit."""
        self.watch_manager.rm_watch(self.watch_manager_dict.values())
        self.notifier.stop()
        if debug: print "[watch]: Watcher thread stopped."

    def unicode_path(self, device):
        """Unicodes the device path to escape problems.

        @param device: The path to the device to be watched.
        @type device: String
        """
        self.path = unicode(device, sys.getfilesystemencoding())


class HandleEvents(pyinotify.ProcessEvent):
    """Subclass for handling the events watched for.

    Interacts with the main application via the "camera" object.
    This object is set by the main application.
    Thread aware. Uses "on", "off" and "none" for the settings in each method.
    """

    def process_IN_CREATE(self, event):
        if debug: print '\t', event.pathname, ' -> was created.'
        self.update_gui("off")

    def process_IN_MOVED_TO(self, event):
        if debug: print '\t', event.pathname, ' -> was moved to here.'
        self.update_gui("off")

    def process_IN_DELETE(self, event):
        if debug: print '\t', event.pathname, ' -> was deleted.'
        self.update_gui("none")

    def process_IN_MOVED_FROM(self, event):
        if debug: print '\t', event.pathname, ' -> was moved to another location.'
        self.update_gui("none")

    def process_IN_DELETE_SELF(self, event):
        if debug: print '\t', event.pathname, ' -> was self deleted.'
        self.update_gui("none")

    def process_IN_OPEN(self, event):
        if debug: print '\t', event.pathname, ' -> was opened.'
        self.update_gui("on")

    def process_IN_CLOSE_WRITE(self, event):
        if debug: print '\t', event.pathname, ' -> was closed (write).'
        self.update_gui("off")

    def process_IN_CLOSE_NOWRITE(self, event):
        if debug: print '\t', event.pathname, ' -> was closed (nowrite).'
        self.update_gui("off")

    def process_default(self, event):
        if debug: print '[watch]: Unhandled event: ', event.maskname

    def update_gui(self, state):
        """Updates the icon and sends notify when video device is changed.

        This is thread aware.
        """
        gtk.gdk.threads_enter()

        camera.device_status = state
        camera.update_icon()

        gtk.gdk.threads_leave()


if __name__ == "__main__":
    print "Description: \n%s" % __doc__
    print "Version: %s" % __version__
    print "Authors: %s" % __authors__
